@@ -5,18 +5,19 @@
 # Fast pinpoint regression test. No options combination nor checksum
 # verification
 
-source "$TOP/tests/common"
-source "$TOP/tests/common.convert"
+source "$TEST_TOP/common" || exit
+source "$TEST_TOP/common.convert" || exit
 
 if ! check_kernel_support_reiserfs >/dev/null; then
 	_not_run "no reiserfs support"
 fi
 
-setup_root_helper
-prepare_test_dev
 check_prereq btrfs-convert
 check_global_prereq mkreiserfs
 
+setup_root_helper
+prepare_test_dev
+
 default_mkfs="mkreiserfs -b 4096"
 convert_test_preamble '' 'large hole extent test' 16k "$default_mkfs"
 convert_test_prep_fs reiserfs $default_mkfs
